import es
import os
import cPickle
import services
import gamethread

info = es.AddonInfo()
info.name = 'Player Access Control'
info.basename = 'player_control'
info.author = 'GODJonez'
info.description = 'Restricts players from playing on your server'
info.version = '1.1'

def requirements():
    if not services.isRegistered('auth'):
        es.dbgmsg(0, 'Cannot load %s - AUTH provider needed'%info.name)
        return False

    return True

pc_data = {
    'restricted': False,
    'players': {},
    }

kickmsg = 'Exclusive access only. Ask server admin for access.'

AUTHCAP = 'change_player_access'

pickle_protocol = 2

lastkicked = None
active_admins = set()
active_players = set()

datafile = os.path.join(es.getAddonPath(info.basename), 'pc_data.p%dd'%pickle_protocol)

def load():
    global pc_data, auth
    if not requirements():
        es.soon('es_xunload %s'%info.basename)
        return False
    auth = services.use('auth')
    auth.registerCapability(AUTHCAP, auth.ADMIN)
    es.addons.registerClientCommandFilter(_ccf)
    
    try:
        df = open(datafile, 'rb')
        pc_data = cPickle.load(df)
        df.close()
        es.dbgmsg(0, '%s database loaded'%info.name)
    except IOError:
        save_pc_data()
        es.dbgmsg(0, '%s database created'%info.name)

    if not es.exists('command', 'pc_allow'):
        es.regcmd('pc_allow', 'player_control/concmd_allow')
        es.regcmd('pc_disallow', 'player_control/concmd_disallow')
        es.regcmd('pc_restrict', 'player_control/concmd_restrict')
        es.regcmd('pc_list', 'player_contrl/concmd_list')
    
    if pc_data['restricted']:
        es.dbgmsg(0, 'Server is restricted for known players only')
    else:
        es.dbgmsg(0, 'Server is open for all players')

    check_for_users()
    active_players.clear()
    active_players.update(es.getUseridList())
    active_admins.clear()
    active_admins.update(filter(lambda userid: auth.isUseridAuthorized(userid, AUTHCAP),active_players))

def unload():
    es.addons.unregisterClientCommandFilter(_ccf)
    
def save_pc_data():
    df = open(datafile, 'wb')
    cPickle.dump(pc_data, df, pickle_protocol)
    df.close()

def check_for_users():
    for userid in es.getUseridList():
        steamid = es.getplayersteamid(userid)
        if steamid not in pc_data['players']:
            _kickuser(userid, es.getplayername(userid), steamid)

def player_activate(ev):
    steamid = ev['es_steamid']
    name = ev['es_username']
    userid = int(ev['userid'])
    if userid not in active_players:
        if steamid not in pc_data['players']:
            if pc_data['restricted']:
                _kickuser(userid, name, steamid)
                return
            else:
                es.msg('New player %s joined the game!'%name)
        else:
            custom = pc_data['players'][steamid]
            gamethread.delayed(5, es.msg, '%s [%s] joined the game!'%(name, custom))
        if auth.isUseridAuthorized(userid, AUTHCAP):
            active_admins.add(userid)
            gamethread.delayed(10, _announce_admin, userid)
        active_players.add(userid)

def player_disconnect(ev):
    userid = int(ev['userid'])
    if userid in active_players:
        active_players.remove(userid)
    if userid in active_admins:
        active_admins.remove(userid)

def _announce_admin(userid):
    es.tell(userid, 'You are authorized to control player access')
    es.tell(userid, 'Commands are:')
    es.tell(userid, 'pc_allow | pc_disallow | pc_restrict | pc_list')

def _kickuser(userid, name, steamid):
    es.server.queuecmd('banid 1 %d'%userid)
    es.server.queuecmd('kickid %d "%s"'%(userid, kickmsg))
    # sometimes kicking doesn't work, so let's check it after short time:
    gamethread.delayed(.5, _check_kick, (userid, name, steamid))

def _check_kick(userid, name, steamid):
    global lastkicked
    if userid not in es.getUseridList():
        _adminmsg('Player %s (%s) kicked: not authorized to play'%(
            name, steamid))
        lastkicked = steamid
    else:
        # try again...
        _kickuser(userid, name, steamid)

def _adminmsg(text):
    es.dbgmsg(0, text)
    for admin in active_admins:
        es.tell(admin, text)
    
def _ccf(userid, args):
    if len(args) == 0:
        return True
    if args[0] == 'pc_allow':
        if auth.isUseridAuthorized(userid, AUTHCAP):
            _allow_player(userid, args[1:])
        else:
            es.cexec(userid, 'echo Restricted command: pc_allow')
        return False
    if args[0] == 'pc_disallow':
        if auth.isUseridAuthorized(userid, AUTHCAP):
            _disallow_player(userid, args[1:])
        else:
            es.cexec(userid, 'echo Restricted command: pc_disallow')
        return False
    if args[0] == 'pc_restrict':
        if auth.isUseridAuthorized(userid, AUTHCAP):
            _toggle_restrict()
        else:
            es.cexec(userid, 'echo Restricted command: pc_restrict')
        return False
    if args[0] == 'pc_list':
        if auth.isUseridAuthorized(userid, AUTHCAP):
            _list_known_players(userid, es.tell)
        else:
            es.cexec(userid, 'echo Restricted command: pc_list')
        return False
    return True

def concmd_allow():
    args = _get_cmd_args()
    if not _allow_player(0, args):
        es.dbgmsg(0, 'Could not find a matching user')

def concmd_disallow():
    args = _get_cmd_args()
    userid = 0
    if 2 in active_admins:
        userid = 2
    if not _disallow_player(userid, args):
        es.dbgmsg(0, 'Could not disallow user')

def concmd_restrict():
    _toggle_restrict()

def concmd_list():
    _list_known_players(0, es.dbgmsg)

def _get_cmd_args():
    argc = es.getargc()
    args = []
    for x in range(argc):
        args.append(es.getargv(x+1))
    return args

def _allow_player(callerid, args):
    custom = 'VIP'
    allowid = ''
    if len(args) == 0:
        allowid = lastkicked
    else:
        arg1 = args[0]
        if 'STEAM_' in arg1 and ':' in arg1:
            allowid = arg1
        elif arg1.isdigit():
            allowid = es.getplayersteamid(arg1)
        else:
            userid = es.getuserid(arg1)
            if userid:
                allowid = es.getplayersteamid(userid)
            else:
                allowid = lastkicked
                custom = arg1
        if len(args) > 1:
            custom = ' '.join(args[1:]).strip()
    if allowid:
        pc_data['players'][allowid] = custom
        es.msg('SteamID %s is now known as [%s] and is allowed to play'%(
            allowid, custom)
               )
        save_pc_data()
        return True
    else:
        es.tell(callerid, 'Could not find a matching user')
        return False

def _disallow_player(callerid, args):
    allowid = ''
    if len(args) >= 1:
        arg1 = args[0]
        if 'STEAM_' in arg1 and ':' in arg1:
            allowid = arg1
        elif arg1.isdigit():
            allowid = es.getplayersteamid(arg1)
        else:
            userid = es.getuserid(arg1)
            if userid:
                allowid = es.getplayersteamid(userid)
    if allowid:
        if allowid in pc_data['players']:
            del pc_data['players'][allowid]
            es.msg('SteamID %s no longer allowed on this server'%allowid)
            if pc_data['restricted']:
                userid = es.getuserid(allowid)
                if userid:
                    _kickuser(userid, es.getplayername(userid), allowid)
            save_pc_data()
            return True
        else:
            es.tell(callerid, 'SteamID %s was not allowed earlier!'%allowid)
    else:
        es.tell(callerid, 'Could not find a matching user')
    return False

def _toggle_restrict():
    if pc_data['restricted']:
        pc_data['restricted'] = False
        es.msg('Server is now open to everyone')
    else:
        pc_data['restricted'] = True
        es.msg('Server is now restricted to only known players')
        check_for_users()
    save_pc_data()
    return True

def _list_known_players(userid, func):
    for steamid in pc_data['players']:
        text = '%s [%s]'%(steamid, pc_data['players'][steamid])
        func(userid, text)
